#SET WORKING DIRECTORY
#update working path accordingly!!!
setwd('/Users/Adam/Desktop/statlab/')

#LOAD REQUIRED LIBRARIES
#install packages if necessary (use install.packages('librarynamehere'))
library(rgdal)
library(spdep)
library(RColorBrewer)
library(classInt)
library(lmtest)

#DEFINE PROJ4 STRINGS
proj.old <- '+proj=longlat +datum=WGS84'
proj.new <-'+proj=aea +lat_1=29.5 +lat_2=45.5 +lat_0=37.5 +lon_0=-96 
        +x_0=0 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=us-mi +no_defs'

#READ AND PROJECT SPATIAL DATA
us <- readOGR(dsn = 'ncovr', layer = 'NAT', p4s = proj.old)
us.proj <- spTransform(us, CRS(proj.new))

#CHOROPLETH MAP
#colors
colors <- brewer.pal(n = 4, name = 'YlOrBr')
classes <- classIntervals(var = us.proj$HR60, n = 4, 
                          style = 'quantile', intervalClosure = 'right')
hr.col <- findColours(clI = classes, pal = colors)

#labels
clabs <- NA
for (i in 1:4){
  clabs[i] <- paste(paste0(round(classes$brks, 2))[i:(i + 1)], collapse = ' - ')  
}

#PLOT IT!
plot(us)
plot(us.proj)
plot(us.proj, col = hr.col)
title(main = 'County Homicide Rates, 1960')
legend(x = 'bottomleft', legend = clabs, fill = colors, 
       xpd = NA, bty = 'n', inset = c(-.12, 0))

#GENERATE KNN WEIGHTS (K = 10)
xy <- coordinates(us.proj)
knn10 <- knearneigh(x = xy, k = 10)
knn10.nb <- knn2nb(knn = knn10)
knn10.listw <- nb2listw(neighbours = knn10.nb)
knn10.mat <- nb2mat(neighbours = knn10.nb)

#ALTERNATIVE WEIGHT SPECIFICATIONS
#contiguity (queen and rook as nb objects)
queen.nb <- poly2nb(pl = us.proj)
rook.nb <- poly2nb(pl = us.proj, queen = FALSE)

#inverse distance (as matrix)
d.mat <- as.matrix(x = dist(xy, upper = TRUE))
invd.mat <- 1/d.mat
diag(invd.mat) <- 0
invd.mat.norm <- invd.mat / rowSums(invd.mat)

#GLOBAL MORAN TEST
#monte carlo test
gmi.mc <- moran.mc(x = us.proj@data$HR60, listw = knn10.listw, nsim = 1000)
print(gmi.mc)

#parametric test
gmi.test <- moran.test(x = us.proj@data$HR60, listw = knn10.listw)
print(gmi.test)

#MORAN SCATTERPLOT
#automated
moran.plot(x = us.proj@data$HR60, 
           listw = knn10.listw)

#by hand
z <- scale(us.proj@data$HR60)
Wz <- knn10.mat %*% z

#you can also do the following to get Wz:
#Wz <- lag.listw(knn10.listw, z)

plot(Wz~z, 
     pch = 16,
     col = 'grey75',
     xlab = 'Homicide Rate (z)',
     ylab = 'Spatially Lagged Homicide Rate (Wz)',
     main = 'Moran Scatterplot of County Homicide Rates, 1960')
moran.mod <- lm(Wz~z, data = us.proj@data)
abline(reg = moran.mod, col = 'red', lwd = 2)
abline(h = 0, v = 0, lwd = 1.5, lty = 2)

#LOCAL MORAN
#Note that the legend position moves if you  change the size of the graph 
#window.In practice, you will want to save the graph directly to a file,
#in which case you will already worked the position out.
lmi <- localmoran(x = us@data$HR60, 
                  listw = knn10.listw, alternative = 'two.sided')
lisa.num <- as.numeric(interaction(z > 0, Wz > 0))
lisa.codes <- factor(lisa.num * (lmi[, 'Pr(z != 0)'] < .05), levels = 0:4)
lisa.labs <- c('Not significant', 'Low-Low', 'High-High', 
               'Low-High', 'High-Low')
plot(us.proj, 
     col = c('white', 'blue', 'pink', 'mediumpurple1', 'red')[lisa.codes],
     main = 'LISA Map of County Homicide Rates, 1960')
legend(x = 'bottomleft',
       legend = lisa.labs, 
       fill = c('white', 'blue', 'red', 'mediumpurple1', 'pink'), 
       xpd = NA, bty = 'n', inset = c(-.12, 0))

#OLS (1960)
mod.60 <- HR60 ~ RD60 + PS60 + MA60 + DV60 + UE60 + factor(SOUTH)
ols.60 <- lm(formula = mod.60, data = us.proj@data)
summary(ols.60)

moran.ols.60 <- lm.morantest(ols.60, knn10.listw)
print(moran.ols.60)

LM.ols.60 <- lm.LMtests(ols.60, knn10.listw, test ='all')
print(LM.ols.60)

vform.60 <- update(mod.60, . ~ .*. + I(RD60^2) + I(PS60^2) + I(MA60^2) 
                   + I(DV60^2) + I(UE60^2) + factor(SOUTH))
white.60 <- bptest(formula = ols.60,
                   varformula = vform.60,
                   data = us.proj@data)
print(white.60)

#OLS (1980)
mod.80 <- HR80 ~ RD80 + PS80 + MA80 + DV80 + UE80 + factor(SOUTH)
ols.80 <- lm(formula = mod.80, data = us.proj@data)
summary(ols.80)

moran.ols.80 <- lm.morantest(ols.80, knn10.listw)
print(moran.ols.80)

LM.ols.80 <- lm.LMtests(ols.80, knn10.listw, test ='all')
print(LM.ols.80)

vform.80 <- update(mod.80, . ~ .*. + I(RD80^2) + I(PS80^2) + I(MA80^2) 
                   + I(DV80^2) + I(UE80^2) + factor(SOUTH))
white.80 <- bptest(formula = ols.80,
                   varformula = vform.80,
                   data = us.proj@data)
print(white.80)
